﻿# pgnDrawDist.ps1 displays the Distance to Draw
$FileIn = "C:\Chess\Stockfish\Tcec\Superfinal14\Tcec14ScidSuperfinal.pgn"
$FileEco = "C:\Chess\Scid\Prog\bin\ecoList.txt"
$FileBok = "C:\Chess\Stockfish\Tcec\ScidOpening.txt"
$StockFishRef = "Stockfish 190203"

$templateEco = @'
{[int]Nbr*:1}.{[string]Eco:D32f}_{[string]Result:=}
{[int]Nbr*:7}.{[string]Eco:C34}_{[string]Result:0}
'@

$templateBok = @'
{[string]Eco*:D32f} [{[string]Name:QGD Tarrasch: von Hennig-Schara Gambit, 7.Qxd5 Nc6}]  {[string]Opening:1.d4 d5 2.c4 e6 3.Nc3 c5 4.cxd5 cxd4 5.Qa4+ Bd7 6.Qxd4 exd5 7.Qxd5 Nc6} 
{[string]Eco*:C34} [{[string]Name:KGA: Schallop Defence}]  {[string]Opening:1.e4 e5 2.f4 exf4 3.Nf3 Nf6} 
'@

$templateTag = @'
[{[string]TagName*:Event} "{[string]TagValue:TCEC Season 14 - Superfinal}"]
[{[string]TagName*:Site} "{[string]TagValue:http://tcec.chessdom.com}"]
'@

$templateMove = @'
{[int]Nbr*:23}. {White:e7xd8=Q+} \{{[string]CommentW:+0.44}\}
{Black:e2xd1=R+} \{{[string]CommentB:-0.44}\}

{[int]Nbr*:7}. {White:e4} \{{[string]CommentW:+0.44}\}
{Black:Nxe4} \{{[string]CommentB:+0.44}\}
'@

$strTags, $strMoves, $strMsgB, $strMsgW, $strMsgWin, $strMsgLoss = "", "", "", "", "", ""
$cr = "`r`n"
$nbrGame, $isDraw, $isSfBlack = 0, 0, 0
Get-Content $FileEco | ConvertFrom-String -TemplateContent $templateEco -OutVariable EcoGames | Out-Null
#$EcoGames | ForEach-Object {Write-Output("game " + $_.Nbr.ToString() + " ECO " + $_.Eco + " Result: " + $_.Result)}
#break

Get-Content $FileBok | ConvertFrom-String -TemplateContent $templateBok -OutVariable EcoOpenings | Out-Null
#$EcoOpenings | ForEach-Object {Write-Output("ECO " + $_.Eco + " Name: " + $_.Name + " Opening: " + $_.Opening)}

Get-Content $FileIn | ForEach-Object { # means For each line of text in the PGN where $_ represents the current line
    if ($_ -match "^\[.*\]"){ $strTag = $strTag + $_ + $cr } # Save pairs of [tag-name "tag_value"]
    elseif($_ -match "^\([^)]*\)"){
        #Write-Output("pv = " + $_)
    } # Ignore principal variation starting at the first column.
    elseif ($_ -match "\{opening\}"){} # Ignore opening book
    elseif ($_ -match "^0-1" -or $_ -match "^1-0" -or $_ -match "^1/2-1/2"){ # Game terminator?
        $nbrGame++
        $isSfBlack = $PgnTags.TagValue.IndexOf($StockFishRef) -eq $PgnTags.TagName.IndexOf("Black")
        if ($isDraw) {
            ConvertFrom-String -InputObject $strMoves -TemplateContent $templateMove -OutVariable PgnMoves | Out-Null
            if ($EcoGames.Result.Item($nbrGame-1) -eq "="){
                $strEco = $EcoGames.Eco.Item($nbrGame-1)
            }
            else
            {
                Write-Output("Cannot find a draw in game " + $nbrGame)
                break
            }
            if ($isSfBlack){$idxDraw = $PgnMoves.CommentB.IndexOf("+0.00")} else {$idxDraw = $PgnMoves.CommentW.IndexOf("+0.00")}
            $strMsg = $PgnMoves.Nbr.Item($idxDraw).ToString("00#")
            if ($isSfBlack){$strMsg += " B"} else {$strMsg += " W"}
            $strMsg += " " + $nbrGame.ToString("00#")
            $strMsg += " " + $strEco + "`t"
            $strMsg += $PgnTags.TagValue.Item($PgnTags.TagName.IndexOf("Opening"))
            $idxVar = $PgnTags.TagName.IndexOf("Variation")
            if ($idxVar -gt 0){$strMsg += " " + $PgnTags.TagValue.Item($idxVar)}
            $strMsg += " " + $EcoOpenings.Opening[$nbrGame-1]
            if ($isSfBlack){$strMsgB += $strMsg + $cr} else {$strMsgW += $strMsg + $cr}
            $isDraw, $strMoves = 0, ""
            #break
        }
        else
        {
            $strEco = $EcoGames.Eco.Item($nbrGame-1)
            $idxPlyCount = $PgnTags.TagName.IndexOf("PlyCount")
            if ($idxPlyCount -gt 0){$MoveNum = ([int]$PgnTags.TagValue.Item($idxPlyCount) + 1) -shr 1} else {$MoveNum = 0}
            $strMsg = $MoveNum.ToString("00#")
            if ($isSfBlack){$strMsg += " B"} else {$strMsg += " W"}
            $strMsg += " " + $nbrGame.ToString("00#")
            $strMsg += " " + $strEco + "`t"
            $strMsg += $PgnTags.TagValue.Item($PgnTags.TagName.IndexOf("Opening"))
            $idxVar = $PgnTags.TagName.IndexOf("Variation")
            if ($idxVar -gt 0){$strMsg += " " + $PgnTags.TagValue.Item($idxVar)}
            $strMsg += " " + $EcoOpenings.Opening[$nbrGame-1]
            if ($isSfBlack){$strMsgB += $strMsg + $cr} else {$strMsgW += $strMsg + $cr}
            if ($PgnTags.TagValue.Contains("1-0")){
                if ($EcoGames.Result.Item($nbrGame-1) -ne "1")
                {
                    Write-Output("Cannot find a White win in game " + $nbrGame)
                    break
                }
                if ($isSfBlack){$strMsgLoss += $strMsg + $cr} else {$strMsgWin += $strMsg + $cr}
            }
            else
            {
                if ($EcoGames.Result.Item($nbrGame-1) -ne "0")
                {
                    Write-Output("Cannot find a White loss in game " + $nbrGame)
                    break
                }
                if ($isSfBlack){$strMsgWin += $strMsg + $cr} else {$strMsgLoss += $strMsg + $cr}
            }
        }
    }
    else # Remaining moves in the middle game and the end game
    {
        if (($isDraw -eq 0) -and ($strTag.Length -gt 0)) {
            ConvertFrom-String -InputObject $strTag -TemplateContent $templateTag -OutVariable PgnTags | Out-Null
            if ($PgnTags.TagValue.Contains("1/2-1/2")){ $isDraw = 1}
            $strTag = ""
        }
        if ($isDraw) { $strMoves = $strMoves + $_ + $cr }
    }
}
Write-Output("Mov C Nbr ECO`tWin in #Mov by " + $StockFishRef + " in game #Nbr")
$strMsgWin -split "`n" | Sort-Object

Write-Output("`nMov C Nbr ECO`tLoss in #Mov by " + $StockFishRef + " in game #Nbr")
$strMsgLoss -split "`n" | Sort-Object

Write-Output("`nMov B Nbr ECO`tDraw in #Mov by " + $StockFishRef + " with the (B)lacks in game #Nbr")
$strMsgB -split "`n" | Sort-Object
Write-Output("`nMov W Nbr ECO`tDraw in #Mov by " + $StockFishRef + " with the (W)hites in game #Nbr")
$strMsgW -split "`n" | Sort-Object

# Expected result: PS C:\Users\Chess> C:\Users\Chess\Documents\Powershell\pgnDrawDist.ps1
# Mov C Nbr ECO	Win in #Mov by Stockfish 190203 in game #Nbr
#
# 044 W 016 B06p	Robatsch (modern) defence 1.e4 g6 2.d4 Bg7 3.Nc3 d6 4.Be3 a6
# ...
# Mov B Nbr ECO	Draw in #Mov by Stockfish 190203 with the (B)lacks in game #Nbr
#
# 007 B 061 A40g	Queen's pawn Keres defence
# 007 B 097 A43n	Old Benoni defence
# 008 B 015 B06p	Robatsch (modern) defence
# 008 B 037 A45p	Trompovsky attack (Ruth, Opovcensky opening)
# ...
# 130 B 075 C93	Ruy Lopez closed, Smyslov defence
# 164 B 039 C33	KGA bishop's gambit, Maurian defence
# 
# Mov W Nbr ECO	Draw in #Mov by Stockfish 190203 with the (W)hites in game #Nbr
# 
# 009 W 082 D15y	QGD Slav Tolush-Geller gambit
# 012 W 044 B89	Sicilian Velimirovic attack
# 013 W 094 A34	English symmetrical, three knights system
# ...
# 131 W 030 A80l	Dutch
# 176 W 088 B01d	Scandinavian Pytel-Wade variation